@@ -4,11 +4,13 @@ package JSON::Schema::Draft201909::Annotation;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Contains a single annotation from a JSON Schema evaluation
 
-our $VERSION = '0.012';
+our $VERSION = '0.013';
 
 use 5.016;
 no if "$]" >= 5.031009, feature => 'indirect';
+no if "$]" >= 5.033001, feature => 'multidimensional';
 use Moo;
+use strictures 2;
 use MooX::TypeTiny;
 use Types::Standard 'Str';
 use namespace::clean;
@@ -62,7 +64,7 @@ JSON::Schema::Draft201909::Annotation - Contains a single annotation from a JSON
 
 =head1 VERSION
 
-version 0.012
+version 0.013
 
 =head1 SYNOPSIS
 
