@@ -4,11 +4,13 @@ package JSON::Schema::Draft201909::Result;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Contains the result of a JSON Schema evaluation
 
-our $VERSION = '0.012';
+our $VERSION = '0.013';
 
 use 5.016;
 no if "$]" >= 5.031009, feature => 'indirect';
+no if "$]" >= 5.033001, feature => 'multidimensional';
 use Moo;
+use strictures 2;
 use MooX::TypeTiny;
 use Types::Standard qw(ArrayRef InstanceOf Enum);
 use MooX::HandlesVia;
@@ -41,9 +43,11 @@ has $_.'s' => (
   },
 ) foreach qw(error annotation);
 
+use constant OUTPUT_FORMATS => [qw(flag basic detailed verbose terse)];
+
 has output_format => (
-  is => 'ro',
-  isa => Enum[qw(flag basic detailed verbose)],
+  is => 'rw',
+  isa => Enum(OUTPUT_FORMATS),
   default => 'basic',
 );
 
@@ -65,6 +69,28 @@ sub format {
         : (errors => [ map $_->TO_JSON, $self->errors ]),
     };
   }
+  if ($style eq 'terse') {
+    return +{
+      valid => $self->result,
+      $self->result
+        ? ($self->annotation_count ? (annotations => [ map $_->TO_JSON, $self->annotations ]) : ())
+        : (errors => [ map $_->TO_JSON,
+            grep {
+              my ($keyword, $error) = ($_->keyword, $_->error);
+              not $keyword
+                or (
+                  not grep $keyword eq $_, qw(allOf anyOf if then else dependentSchemas items propertyNames)
+                  and ($keyword ne 'oneOf' or $error ne 'no subschemas are valid')
+                  and (not grep $keyword eq $_, qw(additionalItems unevaluatedItems)
+                    or $error eq 'additional item not permitted')
+                  and (not grep $keyword eq $_, qw(properties patternProperties)
+                    or $error eq 'property not permitted')
+                  and (not grep $keyword eq $_, qw(additionalProperties unevaluatedProperties)
+                    or $error eq 'additional property not permitted'))
+            }
+            $self->errors ]),
+    };
+  }
 
   die 'unsupported output format';
 }
@@ -90,7 +116,7 @@ JSON::Schema::Draft201909::Result - Contains the result of a JSON Schema evaluat
 
 =head1 VERSION
 
-version 0.012
+version 0.013
 
 =head1 SYNOPSIS
 
@@ -129,11 +155,11 @@ Returns an array of L<JSON::Schema::Draft201909::Error> objects.
 
 =head2 output_format
 
-One of: C<flag>, C<basic>, C<detailed>, C<verbose>. Defaults to C<basic>.
+One of: C<flag>, C<basic>, C<detailed>, C<verbose>, C<terse>. Defaults to C<basic>.
 
 =head1 METHODS
 
-=for Pod::Coverage BUILD
+=for Pod::Coverage BUILD OUTPUT_FORMATS
 
 =head2 format
 
